@@ -2,24 +2,24 @@ import sys
 from pathlib import Path
 from datetime import datetime
 import jupytext
-from sphinx.deprecation import RemovedInSphinx40Warning
 import warnings
 
 HERE = Path(__file__).parent
 sys.path.insert(0, str(HERE.parent))
 sys.path.insert(0, str(HERE / "extensions"))
 
-from scirpy import __author__, __version__
+from importlib.metadata import metadata
+
+info = metadata("scirpy")
 
 # ignore Future warnings (which are caused by dependencies)
-warnings.filterwarnings("ignore", category=RemovedInSphinx40Warning)
 warnings.filterwarnings("ignore", category=FutureWarning)
 
 # General information
-project = "scirpy"
-author = __author__
+project = info["Name"]
+author = info["Author"]
 copyright = f"{datetime.now():%Y}, {author}."
-version = __version__
+version = info["Version"]
 
 templates_path = ["_templates"]
 exclude_patterns = ["_build", "Thumbs.db", ".DS_Store", "**.ipynb_checkpoints"]
@@ -38,9 +38,9 @@ extensions = [
     "sphinx.ext.autosummary",
     "sphinx.ext.mathjax",
     "sphinx.ext.napoleon",
-    "sphinx_autodoc_typehints",
     "sphinxcontrib.bibtex",
     "nbsphinx",
+    "sphinx_autodoc_typehints",
     "scanpydoc",
     *[p.stem for p in (HERE / "extensions").glob("*.py")],
 ]
@@ -63,18 +63,18 @@ todo_include_todos = False
 intersphinx_mapping = dict(
     scanpy=("https://scanpy.readthedocs.io/en/stable/", None),
     anndata=("https://anndata.readthedocs.io/en/stable/", None),
-    h5py=("http://docs.h5py.org/en/stable/", None),
+    h5py=("https://docs.h5py.org/en/stable/", None),
     cycler=("https://matplotlib.org/cycler/", None),
     ipython=("https://ipython.readthedocs.io/en/stable/", None),
     leidenalg=("https://leidenalg.readthedocs.io/en/latest/", None),
-    matplotlib=("https://matplotlib.org/", None),
-    numpy=("https://docs.scipy.org/doc/numpy/", None),
+    matplotlib=("https://matplotlib.org/stable", None),
+    numpy=("https://numpy.org/doc/stable/", None),
     pandas=("https://pandas.pydata.org/pandas-docs/stable/", None),
     python=("https://docs.python.org/3", None),
     scipy=("https://docs.scipy.org/doc/scipy/reference/", None),
     seaborn=("https://seaborn.pydata.org/", None),
     sklearn=("https://scikit-learn.org/stable/", None),
-    networkx=("https://networkx.github.io/documentation/networkx-1.10/", None),
+    networkx=("https://networkx.org/documentation/networkx-1.10/", None),
     dandelion=("https://sc-dandelion.readthedocs.io/en/latest/", None),
 )
 
@@ -102,7 +102,7 @@ html_static_path = ["_static"]
 pygments_style = "sphinx"
 html_context = dict(
     display_github=True,  # Integrate GitHub
-    github_user="icbi-lab",  # Username
+    github_user="scverse",  # Username
     github_repo=project,  # Repo name
     github_version="master",  # Version
     conf_py_path="/docs/",  # Path in the checkout to the docs root
@@ -115,16 +115,6 @@ def setup(app):
     pass
 
 
-# -- Supress 'reference target not found' errors ---------------------------------------
-
-# See https://github.com/agronholm/sphinx-autodoc-typehints/issues/38 for more details.
-qualname_overrides = {
-    "scipy.sparse.coo.coo_matrix": "scipy.sparse.coo_matrix",
-    "scirpy.io._datastructures.AirrCell": "scirpy.io.AirrCell",
-    "pandas.core.arrays.categorical.Categorical": "pandas.Categorical",
-    "pandas.core.series.Series": "pandas.Series",
-}
-
 nitpick_ignore = [
     ("py:class", "igraph.Graph"),
     ("py:class", "igraph.Layout"),
@@ -140,4 +130,7 @@ nitpick_ignore = [
     ("py:class", "D.get(k,d), also set D[k]=d if k not in D"),
     ("py:class", "None.  Update D from mapping/iterable E and F."),
     ("py:class", "an object providing a view on D's values"),
+    # Will work once scipy 1.8 is released
+    ("py:class", "scipy.sparse.base.spmatrix"),
+    ("py:class", "scipy.sparse.csr.csr_matrix"),
 ]
